package eu.mhsl.craftattack.spawn.appliances.worldmuseum;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.listener.InventoryOpenListener;
import eu.mhsl.craftattack.spawn.config.ConfigUtil;
import eu.mhsl.craftattack.spawn.config.Configuration;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.command.MoveWorldMuseumVillagerCommand;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.listener.PlayerEntityInteractListener;
import eu.mhsl.craftattack.spawn.util.ChunkUtils;
import eu.mhsl.craftattack.spawn.util.PluginMessage;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.*;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class WorldMuseum extends Appliance {
    public Villager villager;

    public WorldMuseum() {
        super("worldMuseum");
    }

    @Override
    public void onEnable() {
        // sadly, already existing entities can only be found when chunks are loaded, so to reuse the villager, the chunk has to be loaded
        // https://www.spigotmc.org/threads/getting-entity-problem.332246/

        Location location = ConfigUtil.Position.paseLocation(Objects.requireNonNull(localConfig().getConfigurationSection("villagerLocation")));

        try {
            UUID villagerUuid = UUID.fromString(Objects.requireNonNull(localConfig().getString("uuid")));
            ChunkUtils.loadChunkAtLocation(location);
            this.villager = (Villager) Objects.requireNonNull(Bukkit.getWorld("world")).getEntity(villagerUuid);
            Objects.requireNonNull(this.villager);
        } catch (NullPointerException | IllegalArgumentException e) {
            this.villager = createVillagerEntity();
        }

        this.villager.teleport(location);
    }
    public void updateVillagerPosition(Location location) {
        ConfigUtil.Position.writeLocation(
                Objects.requireNonNull(localConfig().getConfigurationSection("villagerLocation")),
                location
        );
        Configuration.saveChanges();

        this.villager.teleport(location);
    }

    public void handleVillagerInteraction(Player player) {
        Bukkit.getLogger().info("Sending" + player.getName() + " to WorldMuseum");
        PluginMessage.connect(player, localConfig().getString("connect-server-name"));
    }

    private Villager createVillagerEntity() {
        Location villagerLocation = ConfigUtil.Position.paseLocation(Objects.requireNonNull(localConfig().getConfigurationSection("villagerLocation")));
        Villager villager = (Villager) villagerLocation.getWorld().spawnEntity(villagerLocation, EntityType.VILLAGER);

        localConfig().set("uuid", villager.getUniqueId().toString());
        Configuration.saveChanges();

        villager.setInvulnerable(true);
        villager.setPersistent(true);
        villager.setGravity(false);
        villager.setAI(false);
        villager.setCollidable(false);
        villager.setCustomNameVisible(true);
        villager.customName(Component.text("Weltenansicht").color(NamedTextColor.GOLD));
        villager.setProfession(Villager.Profession.LIBRARIAN);
        villager.setVillagerType(Villager.Type.SAVANNA);
        return villager;
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new MoveWorldMuseumVillagerCommand());
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(
                new PlayerEntityInteractListener(),
                new InventoryOpenListener()
        );
    }
}
